/* -----------------------------------------------------------
                            max7219.c

   Header zum Softwaremodul fuer eine  Ansteuerung des
   Multpilex IC MAX7219 in Verbindung mit einer
   8x8 LED Matrix (China-Modul)


   Benoetigte Hardware;

            - 8x8 LED Matrix mit MAX7219 IC

   MCU   :  STM8S103F3 / STM8S003F3
   Takt  :  externer Takt 16 MHz


   29.08.17 by R. Seelig
   ----------------------------------------------------------- */

/*
                 Pinout STM8SF3P6 / STM8S003F3P6

                            ------------
UART1_CK / TIM2_CH1 / PD4  |  1     20  |  PD3 / AIN4 / TIM2_CH2 / ADC_ETR
    UART1_TX / AIN5 / PD5  |  2     19  |  PD2 / AIN3
    UART1_RX / AIN6 / PD6  |  3     18  |  PD1 / SWIM
                     NRST  |  4     17  |  PC7 / SPI_MISO
              OSCIN / PA1  |  5     16  |  PC6 / SPI_MOSI
             OSCOUT / PA2  |  6     15  |  PC5 / SPI_CLK
                Vss (GND)  |  7     14  |  PC4 / TIM1_CH4 / CLK_CCO / AIN2
                VCAP (*1)  |  8     13  |  PC3 / TIM1_CH3 /
                Vdd (+Ub)  |  9     12  |  PB4 / I2C_SCL
           TIM2_CH3 / PA3  | 10     11  |  PB5 / I2C_SDA
                            -----------

*1 :  Ist mit min. 1uF gegen GND zu verschalten


Verbindung MAX7219 Matrixmodul zu STM8S:

       MCU STM8S              8x8 LED-Matrixmodul
       Funktion ( Pin )             Funktion
       ------------------------------------------
          PC7 (  )    ...........      Din
          PC6 (  )    ...........      SCK
          PC3 (  )    ...........      CS

Anmerkung: CS ist eine irrefuehrende Bezeichnung, da es der Uebernahmeimpuls
           eingegangener Daten an die Matrix ist.


*/

#ifndef in_max7219
  #define in_max7219

  #include "stm8s.h"
  #include "stm8_init.h"
  #include "stm8_gpio.h"


  // Pinzuordnung und Ein- Ausschaltmakros
  #define m7219_dininit()    PD2_output_init()
  #define m7219_dinset()     PD2_set()
  #define m7219_dinclr()     PD2_clr()

  #define m7219_clkinit()    PD3_output_init()
  #define m7219_clkset()     PD3_set()
  #define m7219_clkclr()     PD3_clr()

  #define m7219_loadinit()   PD4_output_init()
  #define m7219_loadset()    PD4_set()
  #define m7219_loadclr()    PD4_clr()


  // Pins als Ausgaenge setzen
  #define m7219_init_con()   { m7219_dininit();                      \
                               m7219_clkinit();                      \
                               m7219_loadinit(); }


/* -----------------------------------------------------------
                          PROTOTYPEN
   ----------------------------------------------------------- */
  void m7219_init();
  void m7219_clrscr();
  void m7219_col(uint8_t digit, uint8_t value);
  void m7219_setbmp(uint8_t *bmp);


#endif
